@@ -60,7 +60,7 @@ using namespace KDevelop;
 using namespace Python;
 
 
-PyDUChainTest::PyDUChainTest(QObject* parent): QObject(parent), m_pool()
+PyDUChainTest::PyDUChainTest(QObject* parent): QObject(parent)
 {
     assetsDir = QDir(DUCHAIN_PY_DATA_DIR);
     if (!assetsDir.cd("data")) {
@@ -116,7 +116,12 @@ void PyDUChainTest::init()
     kDebug() << "Searching for python files in " << assetModuleDir.absolutePath();
     
     QList<QString> foundfiles = FindPyFiles(assetModuleDir);
-    
+
+    QString correctionFileDir = KStandardDirs::locate("data", "kdevpythonsupport/correction_files/");
+    KUrl correctionFileUrl = KUrl(correctionFileDir + "testCorrectionFiles/example.py");
+    correctionFileUrl.cleanPath();
+    foundfiles.prepend(correctionFileUrl.path());
+
     for ( int i = 0; i < 2; i++ ) {
         // Parse each file twice, to ensure no parsing-order related bugs appear.
         // Such bugs will need separate unit tests and should not influence these.
@@ -159,6 +164,7 @@ ReferencedTopDUContext PyDUChainTest::parse(const QString& code)
 {
     TestFile* testfile = new TestFile(code + "\n", "py", 0, testDir.absolutePath().append("/"));
     createdFiles << testfile;
+
     testfile->parse((TopDUContext::Features) (TopDUContext::ForceUpdate | TopDUContext::AST) );
     testfile->waitForParsed(500);
     
@@ -432,6 +438,54 @@ void PyDUChainTest::testClassVariables()
     }
 }
 
+void PyDUChainTest::testWarnNewNotCls()
+{
+    QFETCH(QString, code);
+    QFETCH(int, probs);
+
+    ReferencedTopDUContext ctx = parse(code);
+    DUChainReadLocker lock;
+    QCOMPARE(ctx->problems().count(), probs);
+}
+
+void PyDUChainTest::testWarnNewNotCls_data()
+{
+    QTest::addColumn<QString>("code");
+    QTest::addColumn<int>("probs");
+
+    QTest::newRow("check_for_new_first_arg_cls") << "class c():\n def __new__(clf, other):\n  pass" << 1;
+    QTest::newRow("check_for_new_first_arg_cls_0") << "class c():\n def __new__(cls, other):\n  pass" << 0;
+    QTest::newRow("check_first_arg_class_self") << "class c():\n def test(seff, masik):\n  pass" << 1;
+    QTest::newRow("check_first_arg_class_self_0") << "class c():\n def test(self, masik):\n  pass" << 0;
+}
+
+void PyDUChainTest::testBinaryOperatorsUnsure()
+{
+    QFETCH(QString, code);
+    QFETCH(QString, type);
+
+    ReferencedTopDUContext ctx = parse(code);
+    DUChainWriteLocker lock;
+    QList<Declaration*> ds = ctx->findDeclarations(QualifiedIdentifier("checkme"));
+    QVERIFY(!ds.isEmpty());
+    Declaration* d = ds.first();
+    QVERIFY(d);
+    QVERIFY(d->abstractType());
+    QCOMPARE(d->abstractType()->toString(), type);
+}
+
+void PyDUChainTest::testBinaryOperatorsUnsure_data()
+{
+    QTest::addColumn<QString>("code");
+    QTest::addColumn<QString>("type");
+
+    QTest::newRow("check_unsure_type_0") << "class c():\n def __mul__(self, other):\n  return int();\nx = c();\nx = 3.5;\ny = 3;\ncheckme = x * y;" << "unsure (float, int)";
+    QTest::newRow("check_unsure_type_1") << "class c():\n def __mul__(self, other):\n  return int();\nx = c();\nx = 3;\ny = 3;\ncheckme = x * y;" << "int";
+    QTest::newRow("check_unsure_type_2") << "class c():\n pass;\nx = c();\nx = 3;\ny = 3;\ncheckme = x * y;" << "int";
+    QTest::newRow("check_unsure_type_3") << "class c():\n pass;\nclass d():\n pass;\nx = c();\nx = d();\ny = 3;\ncheckme = x * y;" << "int";
+}
+
+
 void PyDUChainTest::testFlickering()
 {
     QFETCH(QStringList, code);
@@ -628,7 +682,7 @@ void PyDUChainTest::testRanges()
         AttributeRangeTestVisitor* visitor = new AttributeRangeTestVisitor();
         visitor->searchingForRange = r;
         visitor->searchingForIdentifier = identifier;
-        visitor->visitCode(m_ast);
+        visitor->visitCode(m_ast.data());
         
         QCOMPARE(visitor->found, true);
         delete visitor;
@@ -693,7 +747,7 @@ void PyDUChainTest::testTypes()
     TypeTestVisitor* visitor = new TypeTestVisitor();
     visitor->ctx = TopDUContextPointer(ctx.data());
     visitor->searchingForType = expectedType;
-    visitor->visitCode(m_ast);
+    visitor->visitCode(m_ast.data());
     
     QCOMPARE(visitor->found, true);
 }
@@ -966,7 +1020,7 @@ void PyDUChainTest::testOperators()
     TypeTestVisitor* visitor = new TypeTestVisitor();
     visitor->ctx = TopDUContextPointer(ctx.data());
     visitor->searchingForType = expectedType;
-    visitor->visitCode(m_ast);
+    visitor->visitCode(m_ast.data());
 
     QVERIFY(visitor->found);
 }
